#include "controller.h"
#include <QQmlEngine>

#include <Carmenta\Engine\Tactical.hpp>

using namespace Carmenta::Engine;

Controller::Controller(QObject* parent) : QObject(parent), mapControl()
{
	QQmlEngine::setObjectOwnership(this, QQmlEngine::CppOwnership);
}

Controller& Controller::instance()
{
	static Controller theController;
	return theController;
}

void Controller::initialize(QObject* root)
{
	// Find the map control in the scene.
	mapControl = root->findChild<QtQuickMapControl*>("mapControl");

	// Load a configuration.
	loadConfiguration("carmenta.engine/hello_world.px");

    initializeTools();
}

void Controller::zoomIn()
{
	// Only for 2D.
	View* view = mapControl->view().get();
	if (view && !dynamic_cast<GlobeView*>(view))
	{
		double s = view->nominalScale();
		view->nominalScale(s * 0.5);
		mapControl->updateView();
	}
}

void Controller::zoomOut()
{
	// Only for 2D.
	View* view = mapControl->view().get();
	if (view && !dynamic_cast<GlobeView*>(view))
	{
		double s = view->nominalScale();
		view->nominalScale(s * 2.0);
		mapControl->updateView();
	}
}

void Controller::loadConfiguration(const QString& fileName)
{
	// Load the configuration.
	ConfigurationPtr config = new Configuration(String::fromUtf8(fileName.toUtf8()));

	// Display the first view in the configuration.
	StringCollectionPtr publicNames = config->publicNames();
	for (uint32_t i = 0; i < publicNames->size(); ++i)
	{
		ViewPtr view = dynamic_cast<View*>(config->getPublicObject(publicNames->get(i)).get());
		if (view)
		{
#ifdef Q_OS_ANDROID
			if (!view->drawable()->pixelSizeAdjustments())
			{
				// Add default pixel size adjustments to drawable, to make configurations designed
				// for PC desktop look good on small hand-held Android devices with higher DPI.
				view->drawable()->pixelSizeAdjustments(PixelSizeAdjustments::fromCurrentDpi(0.40));
			}
#endif
			// Set map control view and tool.
			mapControl->view(view);

			// Check if view is a 3D view.
			if (dynamic_cast<GlobeView*>(view.get()))
			{
				StandardTool3DPtr tool = new StandardTool3D();
				mapControl->tool(tool);

				StandardTouchTool3DPtr touchTool = new StandardTouchTool3D();
				mapControl->touchTool(touchTool);
			}
			else
			{
				StandardToolPtr tool = new StandardTool();
				mapControl->tool(tool);

				StandardTouchToolPtr touchTool = new StandardTouchTool();
				mapControl->touchTool(touchTool);
			}

			mapControl->updateView();

			break;
		}
	}

    // Let's retrieve the memory datasets used to store our different drawings
    _drawnObjectsMemDs = dynamic_cast<MemoryDataSet*>(mapControl->view()->findChildObject("DrawnObjectsMemoryDataSet").get());
    _tacticalDrawingsMemDs = dynamic_cast<MemoryDataSet*>(mapControl->view()->findChildObject("TacticalDrawingsMemoryDataSet").get());
}

QObject* Controller_singletontype_provider(QQmlEngine*, QJSEngine*)
{
	return &Controller::instance();
}

void Controller::initializeTools()
{
    // Declaration of the various 2D tools, including touch
    _standardTool = new StandardTool();
    _standardTouchTool = new StandardTouchTool();

    _createTool = new CreateTool(_drawnObjectsMemDs);
    _createTouchTool = new CreateTouchTool(_drawnObjectsMemDs);

    // Adding the feature handler on the create tools
    static_cast<CreateTool*>(_createTool.get())->addFeatureCreatedHandler(featureCreatedHandler, this);
    static_cast<CreateTouchTool*>(_createTouchTool.get())->addFeatureCreatedHandler(featureCreatedHandler, this);

    // Set the tools on the map control
    mapControl->tool(_standardTool);
    mapControl->touchTool(_standardTouchTool);

    // Tactical create tool initialization: custom create mode and dedicated feature handler
    _createTacticalTool = new CreateTool(_tacticalDrawingsMemDs);
    _createTacticalTool->createMode(ToolCreateMode::ToolCreateModeCustom);
    _createTacticalTool->addFeatureCreatedHandler(featureCreatedHandler, this);

    // Tactical create touch tool initialization: custom create mode and dedicated feature handler
    _createTacticalTouchTool = new CreateTouchTool(_tacticalDrawingsMemDs);
    _createTacticalTouchTool->createMode(ToolCreateMode::ToolCreateModeCustom);
    _createTacticalTouchTool->addFeatureCreatedHandler(featureCreatedHandler, this);
}

void Controller::swapToStandardTool()
{
    mapControl->tool(_standardTool);
    mapControl->touchTool(_standardTouchTool);
}

void Controller::swapToCreateTool(Carmenta::Engine::ToolCreateMode& mode, Carmenta::Engine::CreateToolParametersPtr toolParameters)
{
    // Setting the correct mode on our create tool
    static_cast<CreateTool*>(_createTool.get())->createMode(mode);
    static_cast<CreateTool*>(_createTool.get())->parameters(toolParameters);
    static_cast<CreateTouchTool*>(_createTouchTool.get())->createMode(mode);
    static_cast<CreateTouchTool*>(_createTouchTool.get())->parameters(toolParameters);

    // Setting the create tool on the map control
    mapControl->tool(_createTool);
    mapControl->touchTool(_createTouchTool);
}

void Controller::drawOnMap(QString drawingType)
{
    // Deducing the correct create mode from the argument
    ToolCreateMode createMode;
    CreateToolParametersPtr parameters;
    if (drawingType == "point")
    {
        createMode = ToolCreateMode::ToolCreateModePoint;
    }
    else if (drawingType == "line")
    {
        createMode = ToolCreateMode::ToolCreateModeLine;
    }
    else if (drawingType == "polygon")
    {
        createMode = ToolCreateMode::ToolCreateModePolygon;
    }
    else if (drawingType == "circle")
    {
        createMode = ToolCreateMode::ToolCreateModeEllipse;

        auto ellipseParameters = new EllipseCreateToolParameters();
        ellipseParameters->ellipseCreateMode(ToolEllipseCreateModeCircle);
        parameters = ellipseParameters;
    }
    else
    {
        return;
    }

    // Setting the create mode on the tool and swapping it on the map control
    swapToCreateTool(createMode, parameters);
}

void Controller::tacticalDrawOnMap(QString sidc)
{
    String sidcString = sidc.toStdString().c_str();
    _createTacticalTool->parameters(new MilStd2525CCreateToolParameters(sidcString));
    _createTacticalTouchTool->parameters(new MilStd2525CCreateToolParameters(sidcString));

    mapControl->tool(_createTacticalTool);
    mapControl->touchTool(_createTacticalTouchTool);
}
