#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#ifdef Q_OS_ANDROID
# include <QJniEnvironment>
# include <Carmenta/Engine/Android/AndroidUtils.hpp>
#endif
#include "controller.h"
#include "layermodel.h"
#include "tacticalsymbolsmodel.h"

using namespace Carmenta::Engine;

int main(int argc, char *argv[])
{
	// Use desktop OpenGL.
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
	QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGL);
#else
	QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
#endif

	// Enable high dpi scaling, not needed in Qt 6.
#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
	QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
#endif

	QGuiApplication app(argc, argv);

	int result = -1;
	try
	{
#ifdef Q_OS_ANDROID
		// Log warnings and errors to the Android system log.
		Log::addAppender(new DebuggerLogAppender());
		Log::logLevel(LogLevelWarning);

		// Run some initialization code to setup a Carmenta Engine Android application.
		AndroidUtils::preInitialize(QNativeInterface::QAndroidApplication::context(), QJniEnvironment::javaVM());
#endif
		// Initialize Carmenta Engine.
		Runtime::initialize();

		// Enable alpha buffer by default.
		QQuickWindow::setDefaultAlphaBuffer(true);

		// Make sure the non-threaded scene graph renderer is used.
		QtQuickMapControl::enableNonThreadedRenderer();

		// Register controller singleton class.
		qmlRegisterSingletonType<Controller>("com.carmenta", 1, 0, "Controller", Controller_singletontype_provider);

		// Register the C++ QtQuickMapControl in the QML system
		qmlRegisterType<QtQuickMapControl>("com.carmenta", 1, 0, "MapControl");

		// Initialize the application's GUI.
		QQmlApplicationEngine engine;
		const QUrl url("qrc:/HelloWorldQtQuick/main.qml");
		QObject::connect(&engine, &QQmlApplicationEngine::objectCreated,
						&app, [url](QObject *obj, const QUrl &objUrl) {
			if (!obj && url == objUrl)
				QCoreApplication::exit(-1);
		}, Qt::QueuedConnection);

        QQmlContext* ctxt = engine.rootContext();

		// Create a layer model before loading the QML
        LayerModel model{};
		QStandardItemModel* itemModel = &model;
		ctxt->setContextProperty("layerModel", itemModel);

        // Adding controller to QML context
        ctxt->setContextProperty("controller", &Controller::instance());

        // Adding tacticalModel to QML context
        TacticalSymbolsModel* tacticalModel = new TacticalSymbolsModel();
        ctxt->setContextProperty("myTacticalModel", tacticalModel);

		engine.load(url);
		if (!engine.rootObjects().isEmpty())
		{
			// Connect the QML scene with the controller singleton.
			QList<QObject*> rootObjects = engine.rootObjects();
			Controller::instance().initialize(rootObjects.first());
			model.initialize(); // This populates the layer model

			// Show application and wait for it to exit.
			result = app.exec();
		}
	}
	catch (const EngineException& ex)
	{
		// Display error message from exception.
		qCritical() << "HelloWorldQtQuick: " << ex.message().c_str();
	}

	// Clean up Carmenta Engine.
	Runtime::shutdown();

	return result;
}
