import QtQuick
import QtQuick.Controls
import QtQuick.Controls.Fusion
import com.carmenta 1.0

Window {
    visible: true
    width: 800
    height: 480
    title: qsTr("HelloWorldQtQuick")
    property string sidebarColor: "#303030"
    property string alternateColor: "#e0e0e0"
    property string inactiveColor: "#707070"
    property int sidebarWidth: 50

    function togglePanel(panel, checked) {
        if (checked) {
            window.state = panel
        } else {
            window.state = "HIDDEN"
        }
    }

    Item {
        id: window
        anchors.fill: parent
        MapControl {
            objectName: "mapControl"
            width: parent.width - sidebarWidth
            height: parent.height
        }

        Rectangle {
            id: layers
            width: 200
            x: parent.width // - width
            y: 0
            height: parent.height
            color: alternateColor

            TreeView {
                anchors.fill: parent
                // The model needs to be a QAbstractItemModel
                model: layerModel

                delegate: Item {
                    id: treeDelegate

                    readonly property real rowIndex: row
                    readonly property real columnIndex: column

                    implicitWidth: padding + label.x + label.implicitWidth + padding
                    implicitHeight: label.implicitHeight * 1.7

                    readonly property real indent: 20
                    readonly property real padding: 5

                    // Assigned to by TreeView:
                    required property TreeView treeView
                    required property bool isTreeNode
                    required property bool expanded
                    required property int hasChildren
                    required property int depth

                    TapHandler {
                        onTapped: treeView.toggleExpanded(row)
                    }

                    Text {
                        id: indicator
                        visible: treeDelegate.isTreeNode && treeDelegate.hasChildren
                        width: treeDelegate.indent
                        x: (treeDelegate.depth * treeDelegate.indent)
                        padding: 8
                        anchors.verticalCenter: label.verticalCenter
                        text: "▸"
                        rotation: treeDelegate.expanded ? 90 : 0
                    }
                    CheckBox {
                        id: control
                        width: treeDelegate.indent
                        x: (treeDelegate.depth + 1) * treeDelegate.indent
                        checked: model.checkState
                        hoverEnabled: false
                        background: Rectangle {
                            color: alternateColor
                        }
                        anchors.verticalCenter: parent.verticalCenter
                        indicator: ColorImage {
                            x: (parent.width - width) / 2
                            y: (parent.height - height) / 2
                            source: control.checkState === Qt.Checked ? "/eye.svg" : "/eye-off.svg"
                        }
                        onCheckedChanged: {
                            layerModel.onCheckedChanged(model.treeIndex, checked)
                        }
                    }
                    Text {
                        id: label
                        x: treeDelegate.padding + (treeDelegate.isTreeNode ? (treeDelegate.depth + 2) * treeDelegate.indent : 0)
                        width: treeDelegate.width - treeDelegate.padding - x
                        clip: true
                        text: model.display
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }
            }


            states: [
                State {
                    when: (window.state === "LAYERS")
                    PropertyChanges {
                        target: layers
                        x: parent.width - width - sidebarWidth
                    }
                },
                State {
                    when: (window.state !== "LAYERS")
                    PropertyChanges {
                        target: layers
                        x: parent.width - sidebarWidth
                    }
                }
            ]

            transitions: Transition {
                NumberAnimation {
                    properties: "x"
                    easing.type: Easing.InOutQuad
                }
            }
        }


        Rectangle {
            id: draw
            width: 200
            x: parent.width
            y: 0
            height: parent.height
            color: alternateColor

            ListModel {
                id: drawButtonsModel

                ListElement {
                    name: "Draw Point"
                    icon_name: "/map-marker.svg"
                    draw_type: "point"
                }

                ListElement {
                    name: "Draw Line"
                    icon_name: "/vector-line.svg"
                    draw_type: "line"
                }

                ListElement {
                    name: "Draw Polygon"
                    icon_name: "/vector-polygon.svg"
                    draw_type: "polygon"
                }

                ListElement {
                    name: "Draw Circle"
                    icon_name: "/vector-circle.svg"
                    draw_type: "circle"
                }
            }

            ListView {
                id: drawButtonsListView
                model: drawButtonsModel //setting previously defined ListModel as model
                height: 50*drawButtonsModel.count
                width: parent.width

                delegate: Row {
                    height:  50
                    spacing: 5
                    Button {
                        width: 50
                        checkable: true
                        checked: true
                        anchors.verticalCenter: parent.verticalCenter
                        display: AbstractButton.IconOnly
                        icon.source: icon_name //icon source
                        icon.color: sidebarColor
                        background: Rectangle {
                            color: alternateColor
                        }
                        MouseArea {
                            id: mouseAreaDraw
                            width: parent.parent.parent.width
                            height: parent.height
                            onClicked: {
                                controller.drawOnMap(drawButtonsModel.get(index).draw_type)
                            }
                            Rectangle {
                                anchors.fill: mouseAreaDraw
                                color: mouseAreaDraw.pressed? sidebarColor : alternateColor
                                opacity: 0.3
                            }
                        }
                    }
                    Text {
                        text: name //display name
                        anchors.verticalCenter: parent.verticalCenter
                    }
                }
            }

            ListView {
                id: tacticalDrawing
                model: myTacticalModel // tactical model that we will instantiate in C++
                anchors.top: drawButtonsListView.bottom
                height: parent.height - drawButtonsListView.height
                width: parent.width

                delegate: Row {
                   height: 50
                   spacing: 5
                   Button {
                       width: 50
                       checkable: true
                       checked: true
                       display: AbstractButton.IconOnly
                       icon.source: myTacticalModel.requestImage(index) // our model will act as a provider and provide the images
                       icon.cache: false
                       icon.color: sidebarColor
                       background: Rectangle {
                           color: alternateColor
                       }
                       anchors.verticalCenter: parent.verticalCenter

                       MouseArea {
                            id: mouseAreaTacticalDraw
                            width: 200
                            height: parent.height
                            onClicked: {
                               controller.tacticalDrawOnMap(myTacticalModel.getSidcFromIndex(index)) // callback consists in getting the SIDC from the model and forwarding it to the controller
                                        }
                            Rectangle {
                                anchors.fill: mouseAreaTacticalDraw
                                color: mouseAreaTacticalDraw.pressed? sidebarColor : alternateColor
                                opacity: 0.3
                            }
                        }
                   }
                   Text {
                       text: model.display // using the display role of the model to display
                       anchors.verticalCenter: parent.verticalCenter
                   }
               }
            }

            states: [
                State {
                    when: (window.state === "DRAW")
                    PropertyChanges {
                        target: draw
                        x: parent.width - width - sidebarWidth
                    }
                },
                State {
                    when: (window.state !== "DRAW")
                    PropertyChanges {
                        target: draw
                        x: parent.width - sidebarWidth
                    }
                }
            ]

            transitions: Transition {
                NumberAnimation {
                    properties: "x"
                    easing.type: Easing.InOutQuad
                }
            }
        }

        Rectangle {
            id: sidePanel
            x: parent.width - sidebarWidth
            y: 0
            width: sidebarWidth
            height: parent.height
            color: sidebarColor

            ButtonGroup {
                id: buttonGroup
            }

            ListModel {
                id: buttonModel

                ListElement {
                    name: "Layers"
                    icon_source: "/layers.svg"
                    click: "LAYERS"
                }
                ListElement {
                    name: "Draw"
                    icon_source: "/pencil.svg"
                    click: "DRAW"
                }
            }
            ListView {
                anchors.fill: parent
                model: buttonModel

                delegate: Button {

                    height: 50
                    width: 50
                    checkable: true
                    checked: (window.state === click)
                    display: AbstractButton.IconOnly
                    icon.source: icon_source
                    icon.color: (checked ? alternateColor : inactiveColor)
                    onToggled: togglePanel(click, checked)
                    background: Rectangle {
                        color: sidebarColor
                    }
                }
            }
        }
        states: [
            State {
                name: "HIDDEN"
            },
            State {
                name: "LAYERS"
            },
            State {
                name: "DRAW"
            }
        ]
    }
}

/*##^##
Designer {
    D{i:0;formeditorZoom:0.33}
}
##^##*/
