#include "tacticalsymbolitem.h"

using namespace Carmenta::Engine; 

TacticalSymbolItem::TacticalSymbolItem() : _sidcAtom("sidc")
{
}

TacticalSymbolItem::TacticalSymbolItem(const TacticalSymbolItem& item) : _sidcAtom("sidc")
{
    *this = item;
}

TacticalSymbolItem::TacticalSymbolItem(QString sidc, QString description) :
    _sidcAtom("sidc")
{
    setSidc(sidc, description);
}

TacticalSymbolItem::~TacticalSymbolItem()
{
}

TacticalSymbolItem& TacticalSymbolItem::operator=(const TacticalSymbolItem& item)
{
    setSidc(item.getSidc(), item.name);
    return *this;
}

bool TacticalSymbolItem::setSidc(QString sidc, QString description)
{
    SymbolMetadataPtr symbol = m_standard->metadata()->findBySidc(sidc.toStdString().c_str());
    if (symbol == NULL) return false;

    m_symbolMetadata = symbol;
    name = description == "" ? symbol->description().c_str() : description;
    _sidc = sidc;

    // Reset previews (if location type and interaction hint has changed with the new sidc).
    m_previewLegendItem = NULL;
    preview = computePreview(_sidc.toStdString().c_str());
    return true;
}

QImage TacticalSymbolItem::computePreview(String sidc)
{
    if (m_previewLegendItem == NULL) {
        initializePreview();
    }

    if (m_previewLegendItem != NULL) {
        // Setting SIDC attribute value on legend item
        m_previewLegendItem->attributes()->set(_sidcAtom, sidc);
        
        // Generate a raster image using the LegendItem
        ByteArray bytes = m_previewLegendItem->getImage(m_previewLegendItem->width(), m_previewLegendItem->height());
        RasterPtr image = Raster::fromStream(bytes.data(), bytes.size(), ImageFormatPng);

        // Generate the QImage preview from the Raster. Need to flip (mirror) on the Y axis.
        QImage preview(image->data(), image->width(), image->height(), image->bytesPerRow(), QImage::Format::Format_RGBA8888);
        return preview.mirrored();
    }

    return QImage();
}

void TacticalSymbolItem::initializePreview()
{
    // The legend item utilizes a visualizer to obtain an icon
    TacticalVisualizerPtr previewTacticalViz = new TacticalVisualizer();
    MilStd2525CParametersPtr milParameters = new MilStd2525CParameters();
    previewTacticalViz->parameters(milParameters);
    previewTacticalViz->parameters()->fill(new DirectBoolAttributeVariable(false));

    int32_t w = 100, h = 100;
    // Initialize the LegendItem with a tactical visualizer and changes parameters depending on the Symbol's LocationType
    switch (m_symbolMetadata->locationType())
    {
    case LocationTypePoint:
        previewTacticalViz->parameters()->haloColor(new DirectColorAttributeVariable(Color(255, 255, 255, 180)));
        m_previewLegendItem = new PointLegendItem();
        break;

    case LocationTypeFanArea:
    {
        m_previewLegendItem = new PointLegendItem();
        switch (m_symbolMetadata->interactionHint())
        {
        case InteractionHintFanAreaCircle:
        case InteractionHintFanAreaCircleOrientation:
            m_previewLegendItem->attributes()->set("fanAreaSectorSize", 360.0);
            m_previewLegendItem->attributes()->set("fanAreaOrientation", 0.0);
            m_previewLegendItem->attributes()->set("fanAreaMinimumRange", 0.0);
            m_previewLegendItem->attributes()->set("fanAreaMaximumRange", 30.0);
            break;
        case InteractionHintFanAreaSector:
        case InteractionHintFanArea3Points:
            w = 240;
            h = 240;
            milParameters->width(new DirectDoubleAttributeVariable(4.0));
            m_previewLegendItem->attributes()->set("fanAreaSectorSize", 60.0);
            m_previewLegendItem->attributes()->set("fanAreaOrientation", 0.0);
            m_previewLegendItem->attributes()->set("fanAreaMinimumRange", 100.0);
            m_previewLegendItem->attributes()->set("fanAreaMaximumRange", 100.0);
            break;
        default:
            break;
        }
        break;
    }

    case LocationTypeLine:
        m_previewLegendItem = new LineLegendItem();
        break;

    case LocationTypeCorridorArea:
        m_previewLegendItem = new LineLegendItem();
        m_previewLegendItem->attributes()->set("corridorAreaWidth", 30.0);
        break;

    case LocationTypeOrbitArea:
        m_previewLegendItem = new LineLegendItem();
        m_previewLegendItem->attributes()->set("orbitAreaWidth", 20.0);
        m_previewLegendItem->attributes()->set("orbitAreaAlignment", Atom("Right"));
        break;

    case LocationTypePolygonArea:
    {
        w = 180;
        h = 180;
        PolygonLegendItemPtr poly = new PolygonLegendItem();
        poly->polygonType(LegendPolygonTypeLake);
        m_previewLegendItem = poly;
        milParameters->width(new DirectDoubleAttributeVariable(4.0));
        
        break;
    }

    default:
        break;
    }

    // Let's set the visualizer on the legend item as well as a target size
    if (m_previewLegendItem != NULL) {
        m_previewLegendItem->width(w);
        m_previewLegendItem->height(h);
        m_previewLegendItem->visualizers()->add(previewTacticalViz);
    }
}